'use strict';
const sinon = require('sinon');

const chai = require('chai');
chai.use(require('sinon-chai'));
const expect = chai.expect;


const Adapter = require('../../');
const baseAdapter = require('sails-postgresql');

describe('find', function () {
  let testRecordList, options, connection, collections;

  before(done => {
    sinon.stub(baseAdapter, 'registerConnection').callsArgWith(2, null);
    connection = {
      identity: 'findTestConnection'
    };
    collections = {
      testTable: {
        id: 'testTable'
      }
    };
    Adapter.registerConnection(connection, collections, done);
  });

  after(() => {
    baseAdapter.registerConnection.restore();
  });

  beforeEach(() => {
    options = {
      select: ['id'],
      orderBy: 'id'
    };
    testRecordList = [{
      id: 1
    }, {
      id: 2
    }];
    sinon.stub(baseAdapter, 'find').callsArgWith(3, null, testRecordList);
    collections.testTable.afterFind = sinon.stub().callsFake((v, cb) => cb(null, v));
  });

  afterEach(() => {
    baseAdapter.find.restore();
  });

  it('should call baseAdapter.find and callback with the response', function (done) {
    Adapter.find('findTestConnection', 'testTable', options, (err, recordList) => {
      expect(baseAdapter.find).to.be.calledWith('findTestConnection', 'testTable', options, sinon.match.func);
      expect(err).not.to.exist;
      expect(recordList).to.eql(testRecordList);
      done();
    });
  });

  it('should call collection afterFind lifecycle hook on each record', function (done) {
    Adapter.find('findTestConnection', 'testTable', options, (err, recordList) => {
      recordList.map(rec => expect(collections.testTable.afterFind).calledWith(rec, sinon.match.func).calledAfter(baseAdapter.find));
      expect(err).not.to.exist;
      expect(recordList).to.eql(testRecordList);
      done();
    });
  });

  it('should error and not call afterFind if baseAdapter.find errors', function (done) {
    baseAdapter.find.callsArgWith(3, 'testError', testRecordList);
    Adapter.find('findTestConnection', 'testTable', options, (err, recordList) => {
      expect(baseAdapter.find).calledWith('findTestConnection', 'testTable', options, sinon.match.func);
      expect(err).to.eql('testError');
      expect(recordList).to.eql(testRecordList);
      expect(collections.testTable.afterFind).not.to.be.called;
      done();
    });
  });
});