'use strict';
const sinon = require('sinon');

const chai = require('chai');
chai.use(require('sinon-chai'));
const expect = chai.expect;


const Adapter = require('../../');
const baseAdapter = require('sails-postgresql');

describe('registerConnection', function () {
  let connInfo;
  let collections;

  beforeEach(() => {
    connInfo = {
      identity: 'testConnection'
    };

    collections = {
      testColl: {
        id: 'testColl'
      }
    };
    
    sinon.stub(baseAdapter, 'registerConnection').callsArgWith(2, null, 'testResp');
  });

  afterEach(() => {
    baseAdapter.registerConnection.restore();
  });

  it('should call baseAdapter.registerConnection and callback with the response', function (done) {
    Adapter.registerConnection(connInfo, collections, (err, resp) => {
      expect(baseAdapter.registerConnection).calledWith(connInfo, collections, sinon.match.func);
      expect(err).not.to.exist;
      expect(resp).to.eql('testResp');
      done();
    });
  });

  it('should error if baseAdapter.registerConnection errors', function (done) {
    baseAdapter.registerConnection.callsArgWith(2, 'test-error', 'test-resp');
    Adapter.registerConnection(connInfo, collections, (err, resp) => {
      expect(baseAdapter.registerConnection).calledWith(connInfo, collections, sinon.match.func);
      expect(err).to.eql('test-error');
      expect(resp).to.eql('test-resp');
      done();
    });
  });

  it('should error if collections parameter is array', function (done) {
    Adapter.registerConnection(connInfo, [collections.testColl], (err, resp) => {
      expect(err.message).to.include("'collections' parameter is an array, expected object");
      done();
    });
  });
});